from importlib.metadata import version, PackageNotFoundError
from colorama import Fore
import requests

def help() -> str:
    return f"""
{Fore.RED}Tonviewer{Fore.WHITE} - TON Wallet & Crypto Info CLI Tool
Usage:
        {Fore.RED}Tonviewer {Fore.WHITE}-[OPTIONS] "[FOR-OPTION]"
Options:
  {Fore.RED}-w, --wallet{Fore.WHITE}          Print wallet balance.
  {Fore.RED}-i, --info{Fore.WHITE}            Get wallet info.
  {Fore.RED}-t, --transactions{Fore.WHITE}    Get wallet transactions.
  {Fore.RED}-l, --limit{Fore.WHITE}           Number of transactions to fetch.
  {Fore.RED}-H, --hashtx{Fore.WHITE}          Pass a transaction hash.
  {Fore.RED}-v, --version{Fore.WHITE}         Show Tonviewer version.
  {Fore.RED}-h, --help{Fore.WHITE}            Show this help message.
"""


def versions():
    try:
        pkg_version = version("tonviewer")
    except PackageNotFoundError:
        pkg_version = "unknown"

    return f"""
{Fore.RED}Tonviewer {Fore.WHITE}Version: {Fore.RED}{pkg_version}{Fore.WHITE}
Author: {Fore.RED}PyCodz{Fore.WHITE} .

({Fore.RED}PyPI{Fore.WHITE})      : https://pypi.org/project/Tonviewer .
({Fore.RED}GitHub{Fore.WHITE})    : https://github.com/DevZ44d/Tonviewer.git .
({Fore.RED}Telegram{Fore.WHITE})  : https://t.me/PyCodz .
({Fore.RED}Portfolio{Fore.WHITE}) : https://deep.is-a.dev .
"""

def check_for_update(package_name: str):
    try:
        current_version = version(package_name)
        url = f"https://pypi.org/pypi/{package_name}/json"
        latest_version = requests.get(url).json()["info"]["version"]

        if current_version != latest_version:
            print(f"""{Fore.WHITE}
[{Fore.BLUE}notice{Fore.WHITE}] New release available: {Fore.RED}{current_version}{Fore.WHITE} → {Fore.GREEN}{latest_version}{Fore.WHITE}
[{Fore.BLUE}notice{Fore.WHITE}] Run: {Fore.GREEN}python -m pip install --upgrade {package_name}{Fore.WHITE}
""")
    except Exception:
        pass




from .INFO.wallet import IN
from .Hash.main import EventResolver
from .Transaction.main import Transactions
from typing import Optional
from .main import check_for_update

class Wallet:
    def __init__(self, wallet: Optional[str]) -> None:
        self.wallet = wallet

    def transactions(self, limit: Optional[int]):
        tr = Transactions(self.wallet)
        print(tr.get(limit=limit))
        check_for_update("tonviewer")

    def info(self):
        inf = IN(wallet=self.wallet)
        print(inf.balance())
        check_for_update("tonviewer")


class HashTx:
    def __init__(self, hashtx: Optional[str]) -> None:
        self.hashtx = hashtx

    def get(self):
        h = EventResolver(event_address=self.hashtx)
        print(h.result())
        check_for_update("tonviewer")


import argparse
from .file import Wallet, HashTx
from .main import help, versions
from colorama import Fore

def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser(
        description="Tonviewer is a Python library to fetch wallet info, transactions, and transaction hash data.",
        add_help=False
    )

    parser.add_argument("-w", "--wallet", type=str, help="Wallet address")
    parser.add_argument("-i", "--info", action="store_true", help="Get wallet info")
    parser.add_argument("-t", "--transactions", type=str, help="Wallet address for transactions")
    parser.add_argument("-l", "--limit", type=int, help="Number of transactions to fetch")
    parser.add_argument("-H", "--hashtx", type=str, help="Transaction hash to query")
    parser.add_argument("-h", "--help", action="store_true", help="Display help message")
    parser.add_argument("-v", "--version", action="store_true", help="Display version info")

    args, unknown = parser.parse_known_args()
    if unknown:
        for arg in unknown:
            print(f"[{Fore.RED}ERROR{Fore.WHITE}] No valid arguments provided. Use `{Fore.RED}-h{Fore.WHITE}` for help.")
        exit(1)

    return args

def run_cli():
    args = parse_args()

    try:
        # Help / Version
        if args.help:
            print(help())
            return

        if args.version:
            print(versions())
            return

        # Transaction hash
        if args.hashtx:
            HashTx(hashtx=args.hashtx).get()
            return

        # Wallet info
        if args.wallet and args.info:
            Wallet(wallet=args.wallet).info()
            return

        # Transactions
        if args.transactions:
            limit = args.limit if args.limit and args.limit > 0 else None
            Wallet(wallet=args.transactions).transactions(limit)
            return

        # Default error
        print(f"[{Fore.RED}ERROR{Fore.WHITE}] No valid arguments provided. Use `{Fore.RED}-h{Fore.WHITE}` for help.")

    except Exception as e:
        print(f"[FATAL] Unexpected error: {e}")

def console():
    run_cli()

if __name__ == "__main__":
    console()


import requests
import datetime
import json
from typing import List, Dict, Any
from tonsdk.utils import Address
from ..dollers import Dollers


class Transactions:
    """
    Fetch and format TON wallet transactions using TONAPI events.
    """

    TON_DECIMALS = 1_000_000_000
    def __init__(self, wallet: str):
        self.wallet = wallet
        self.base_url = "https://tonapi.io/v2"
        self._dollar_rate = Dollers()._TON_USDT()


    @staticmethod
    def raw_to_UQ(raw: str) -> str:
        try:
            addr = Address(raw)
            return addr.to_string(
                is_user_friendly=True,
                is_url_safe=True,
                is_bounceable=False,
                is_test_only=False
            )
        except Exception:
            return raw


    @staticmethod
    def format_time(timestamp: int) -> str:
        return datetime.datetime.fromtimestamp(
            timestamp
        ).strftime("%d %b %I:%M %p")

    @staticmethod
    def clean_comment(comment: str) -> str:
        if not comment:
            return ""

        if "Ref#" in comment:
            comment = comment.split("Ref#")[0]

        return comment.strip()

    def get(self, limit: int = 1) -> str:

        url = f"{self.base_url}/accounts/{self.wallet}/events?limit={limit}"

        try:
            response = requests.get(url)
            response.raise_for_status()
            data = response.json()

            results: List[Dict[str, Any]] = []
            counter = 1

            for event in data.get("events", []):

                actions = event.get("actions", [])


                ton_transfers = [
                    a for a in actions
                    if a.get("type") == "TonTransfer"
                ]

                if not ton_transfers:
                    continue

                event_time = self.format_time(event["timestamp"])

                for action in ton_transfers:

                    transfer = action["TonTransfer"]

                    amount = int(transfer["amount"]) / self.TON_DECIMALS
                    usd_value = amount * self._dollar_rate

                    sender = self.raw_to_UQ(
                        transfer["sender"]["address"]
                    )
                    recipient = self.raw_to_UQ(
                        transfer["recipient"]["address"]
                    )

                    action_type = (
                        "Received TON"
                        if recipient == self.wallet
                        else "Sent TON"
                    )

                    comment = self.clean_comment(
                        transfer.get("comment", "")
                    )

                    tx: Dict[str, Any] = {
                        "Time": event_time,
                        "Action": action_type,
                        "From": sender,
                        "To": recipient,
                        "Paid For": comment if comment else None,
                        "Price": (
                            f"{'-' if action_type=='Sent TON' else '+'}"
                            f"{amount:.3f} TON ≈ {usd_value:.5f} $"
                        ),
                        "Limit": str(counter)
                    }
                    counter += 1

                    results.append(tx)



            cleaned = [
                {k: v for k, v in tx.items() if v is not None}
                for tx in results
            ]

            skipped = limit - (counter - 1)

            output = {
                "transactions": cleaned,
                "skipped": f"Skipped {skipped} NFT Transactions"
            }

            return json.dumps(output, indent=2, ensure_ascii=False)



        except requests.RequestException:
            return json.dumps(
                {"error": "Failed to fetch transactions"},
                indent=2
            )





import requests
import json
import datetime
from typing import Dict, Any, Optional, List
from tonsdk.utils import Address
from ..dollers import Dollers


class EventResolver:
    TON_DECIMALS = 1_000_000_000

    def __init__(self, *, event_address: str):
        self.event_address = event_address
        self.event_url = f"https://tonapi.io/v2/events/{event_address}"
        self._cached_event: Optional[Dict[str, Any]] = None
        self._doller = Dollers()._TON_USDT()


    def _fetch_event(self) -> Dict[str, Any]:
        if self._cached_event:
            return self._cached_event

        r = requests.get(self.event_url, timeout=10)
        r.raise_for_status()
        self._cached_event = r.json()
        return self._cached_event

    @staticmethod
    def normalize(addr: str, name: Optional[str] = None) -> str:
        if name:
            return name
        try:
            return Address(addr).to_string(
                is_user_friendly=True,
                is_url_safe=True,
                is_bounceable=False,
                is_test_only=False
            )
        except Exception:
            return addr

    @staticmethod
    def format_time(timestamp: int) -> str:
        return datetime.datetime.fromtimestamp(
            timestamp
        ).strftime("%Y-%m-%d %I:%M:%S %p")

    @staticmethod
    def format_action_name(status: str, action_type: str) -> str:
        return f"{status.upper()} {action_type.upper().replace('_', ' ')}"


    def _parse_gas_relay(
        self,
        action: Dict[str, Any],
        timestamp: int
    ) -> Dict[str, Any]:

        relay = action["GasRelay"]
        preview = action.get("simple_preview", {})

        return {
            "Time": self.format_time(timestamp),
            "Action": self.format_action_name(
                action.get("status", "ok"),
                action.get("type", "GasRelay")
            ),
            "From": self.normalize(
                relay["relayer"]["address"],
                relay["relayer"].get("name")
            ),
            "To": self.normalize(
                relay["target"]["address"],
                relay["target"].get("name")
            ),
            "Price": preview.get("value"),
            "Comment": preview.get("description"),
            "Status": "SUCCESS"
        }

    def _parse_jetton_transfer(
        self,
        action: Dict[str, Any],
        timestamp: int
    ) -> Dict[str, Any]:

        transfer = action["JettonTransfer"]
        preview = action.get("simple_preview", {})

        sender = transfer["sender"]
        recipient = transfer["recipient"]

        result = {
            "Time": self.format_time(timestamp),
            "Action": self.format_action_name(
                action.get("status", "ok"),
                action.get("type", "JettonTransfer")
            ),
            "From": self.normalize(
                sender["address"],
                sender.get("name")
            ),
            "To": self.normalize(
                recipient["address"],
                recipient.get("name")
            ),
            "Price": preview.get("value"),
            "Status": "SUCCESS"
        }

        comment = transfer.get("comment")
        if comment:
            result["Comment"] = comment

        return result

    def _parse_generic_action(
        self,
        action: Dict[str, Any],
        timestamp: int
    ) -> Dict[str, Any]:

        preview = action.get("simple_preview", {})
        accounts = preview.get("accounts", [])

        sender = accounts[0] if len(accounts) > 0 else {}
        receiver = accounts[1] if len(accounts) > 1 else {}

        status = action.get("status", "ok")
        action_type = action.get("type", "UNKNOWN")

        return {
            "Time": self.format_time(timestamp),
            "Action": self.format_action_name(status, action_type),
            "From": self.normalize(
                sender.get("address", ""),
                sender.get("name")
            ),
            "To": self.normalize(
                receiver.get("address", ""),
                receiver.get("name")
            ),
            "Price": preview.get("value"),
            "Comment": preview.get("description"),
            "Status": "SUCCESS" if status == "ok" else "FAILED"
        }


    def extract_info(self) -> Dict[str, Any]:
        try:
            data = self._fetch_event()
            timestamp = data.get("timestamp", 0)
            actions = data.get("actions", [])

            if not actions:
                return {"Error": "No actions found"}

            results: List[Dict[str, Any]] = []

            for action in actions:

                action_type = action.get("type")

                if action_type == "GasRelay":
                    results.append(
                        self._parse_gas_relay(action, timestamp)
                    )
                    continue

                if action_type == "JettonTransfer":
                    results.append(
                        self._parse_jetton_transfer(action, timestamp)
                    )
                    continue

                results.append(
                    self._parse_generic_action(action, timestamp)
                )

            cleaned = [
                {k: v for k, v in r.items() if v is not None}
                for r in results
            ]

            return {"actions": cleaned}

        except requests.HTTPError:
            return {
                "Error":
                "Invalid transaction address."
            }

        except Exception as e:
            return {"Error": f"Parsing error: {e}"}

    def result(self) -> str:
        return json.dumps(
            self.extract_info(),
            indent=2,
            ensure_ascii=False
        )
